@@ -40,20 +40,22 @@
 /* Implementation: */
 
 #if defined(__STDC_VERSION__) && __STDC_VERSION__ == 201112L
-	_Thread_local int _libcpuid_errno = ERR_OK;
+#define INTERNAL_SCOPE _Thread_local
 #elif defined(__GNUC__) // Also works for clang
-	__thread int _libcpuid_errno = ERR_OK;
+#define INTERNAL_SCOPE __thread
 #else
-	static int _libcpuid_errno = ERR_OK;
+#define INTERNAL_SCOPE static
 #endif
 
-int set_error(cpu_error_t err)
+INTERNAL_SCOPE int _libcpuid_errno = ERR_OK;
+
+int cpuid_set_error(cpu_error_t err)
 {
 	_libcpuid_errno = (int) err;
 	return (int) err;
 }
 
-int get_error()
+int cpuid_get_error()
 {
 	return _libcpuid_errno;
 }
@@ -124,7 +126,7 @@ static void cpuid_grow_raw_data_array(struct cpu_raw_data_array_t* raw_array, lo
 	debugf(3, "Growing cpu_raw_data_array_t from %u to %u items\n", raw_array->num_raw, n);
 	tmp = realloc(raw_array->raw, sizeof(struct cpu_raw_data_t) * n);
 	if (tmp == NULL) { /* Memory allocation failure */
-		set_error(ERR_NO_MEM);
+		cpuid_set_error(ERR_NO_MEM);
 		return;
 	}
 
@@ -143,7 +145,7 @@ static void cpuid_grow_system_id(struct system_id_t* system, uint8_t n)
 	debugf(3, "Growing system_id_t from %u to %u items\n", system->num_cpu_types, n);
 	tmp = realloc(system->cpu_types, sizeof(struct cpu_id_t) * n);
 	if (tmp == NULL) { /* Memory allocation failure */
-		set_error(ERR_NO_MEM);
+		cpuid_set_error(ERR_NO_MEM);
 		return;
 	}
 
@@ -173,10 +175,29 @@ static int get_total_cpus(void)
 }
 #define GET_TOTAL_CPUS_DEFINED
 
+INTERNAL_SCOPE thread_affinity_policy_data_t saved_affinity;
+
+static bool save_cpu_affinity(void)
+{
+	mach_msg_type_number_t count = THREAD_AFFINITY_POLICY_COUNT;
+	boolean_t get_default = false;
+	return thread_policy_get(mach_thread_self(), THREAD_AFFINITY_POLICY, (thread_policy_t) &saved_affinity, &count, &get_default) == KERN_SUCCESS;
+}
+
+static bool restore_cpu_affinity(void)
+{
+	return thread_policy_set(mach_thread_self(), THREAD_AFFINITY_POLICY, (thread_policy_t) &saved_affinity, THREAD_AFFINITY_POLICY_COUNT) == KERN_SUCCESS;
+}
+#define PRESERVE_CPU_AFFINITY
+
 static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 {
 	thread_affinity_policy_data_t ap;
 	ap.affinity_tag = logical_cpu + 1;
+	/* Note: thread_policy_set() always returns KERN_SUCCESS even if the target logical CPU does not exist
+	   Refer to #178 */
+	if (logical_cpu >= get_total_cpus())
+		return false;
 	return thread_policy_set(mach_thread_self(), THREAD_AFFINITY_POLICY, (thread_policy_t) &ap, THREAD_AFFINITY_POLICY_COUNT) == KERN_SUCCESS;
 }
 #define SET_CPU_AFFINITY
@@ -192,6 +213,52 @@ static int get_total_cpus(void)
 }
 #define GET_TOTAL_CPUS_DEFINED
 
+#if (_WIN32_WINNT >= 0x0601)
+INTERNAL_SCOPE GROUP_AFFINITY savedGroupAffinity;
+#else
+INTERNAL_SCOPE DWORD_PTR savedAffinityMask = 0;
+#endif
+
+static bool save_cpu_affinity(void)
+{
+#if (_WIN32_WINNT >= 0x0601)
+	HANDLE thread = GetCurrentThread();
+	return GetThreadGroupAffinity(thread, &savedGroupAffinity);
+#else
+/* Credits to https://stackoverflow.com/questions/6601862/query-thread-not-process-processor-affinity#6601917 */
+	HANDLE thread = GetCurrentThread();
+	DWORD_PTR threadAffinityMask = 1;
+	while (threadAffinityMask) {
+		savedAffinityMask = SetThreadAffinityMask(thread, threadAffinityMask);
+		if(savedAffinityMask)
+			return SetThreadAffinityMask(thread, savedAffinityMask);
+		else if (GetLastError() != ERROR_INVALID_PARAMETER)
+			return false;
+
+		threadAffinityMask <<= 1; // try next CPU
+	}
+	return false;
+#endif
+}
+
+static bool restore_cpu_affinity(void)
+{
+#if (_WIN32_WINNT >= 0x0601)
+	if (!savedGroupAffinity.Mask)
+		return false;
+
+	HANDLE thread = GetCurrentThread();
+	return SetThreadGroupAffinity(thread, &savedGroupAffinity, NULL);
+#else
+	if (!savedAffinityMask)
+		return false;
+
+	HANDLE thread = GetCurrentThread();
+	return SetThreadAffinityMask(thread, savedAffinityMask);
+#endif
+}
+#define PRESERVE_CPU_AFFINITY
+
 static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 {
 /* Credits to https://github.com/PolygonTek/BlueshiftEngine/blob/fbc374cbc391e1147c744649f405a66a27c35d89/Source/Runtime/Private/Platform/Windows/PlatformWinThread.cpp#L27 */
@@ -225,9 +292,9 @@ static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 		warnf("set_cpu_affinity for logical CPU %u is not supported in this operating system.\n", logical_cpu);
 		return -1;
 	}
-	HANDLE process = GetCurrentProcess();
-	DWORD_PTR processAffinityMask = 1ULL << logical_cpu;
-	return SetProcessAffinityMask(process, processAffinityMask);
+	HANDLE thread = GetCurrentThread();
+	DWORD_PTR threadAffinityMask = 1ULL << logical_cpu;
+	return SetThreadAffinityMask(thread, threadAffinityMask);
 #endif /* (_WIN32_WINNT >= 0x0601) */
 }
 #define SET_CPU_AFFINITY
@@ -258,6 +325,19 @@ static int get_total_cpus(void)
 #if defined linux || defined __linux__
 #include <sched.h>
 
+INTERNAL_SCOPE cpu_set_t saved_affinity;
+
+static bool save_cpu_affinity(void)
+{
+	return sched_getaffinity(0, sizeof(saved_affinity), &saved_affinity) == 0;
+}
+
+static bool restore_cpu_affinity(void)
+{
+	return sched_setaffinity(0, sizeof(saved_affinity), &saved_affinity) == 0;
+}
+#define PRESERVE_CPU_AFFINITY
+
 static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 {
 	cpu_set_t cpuset;
@@ -273,6 +353,19 @@ static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 #include <sys/processor.h>
 #include <sys/procset.h>
 
+INTERNAL_SCOPE processorid_t saved_binding = PBIND_NONE;
+
+static bool save_cpu_affinity(void)
+{
+	return processor_bind(P_LWPID, P_MYID, PBIND_QUERY, &saved_binding) == 0;
+}
+
+static bool restore_cpu_affinity(void)
+{
+	return processor_bind(P_LWPID, P_MYID, saved_binding, NULL) == 0;
+}
+#define PRESERVE_CPU_AFFINITY
+
 static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 {
 	if (logical_cpu > (sizeof(processorid_t) * 8)) {
@@ -303,6 +396,19 @@ static int get_total_cpus(void)
 #include <sys/param.h>
 #include <sys/cpuset.h>
 
+INTERNAL_SCOPE cpuset_t saved_affinity;
+
+static bool save_cpu_affinity(void)
+{
+	return cpuset_getaffinity(CPU_LEVEL_WHICH, CPU_WHICH_TID, -1, sizeof(saved_affinity), &saved_affinity) == 0;
+}
+
+static bool restore_cpu_affinity(void)
+{
+	return cpuset_setaffinity(CPU_LEVEL_WHICH, CPU_WHICH_TID, -1, sizeof(saved_affinity), &saved_affinity) == 0;
+}
+#define PRESERVE_CPU_AFFINITY
+
 static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 {
 	cpuset_t cpuset;
@@ -317,6 +423,19 @@ static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 #include <pthread.h>
 #include <pthread_np.h>
 
+INTERNAL_SCOPE cpuset_t saved_affinity;
+
+static bool save_cpu_affinity(void)
+{
+	return pthread_getaffinity_np(pthread_self(), sizeof(saved_affinity), &saved_affinity) == 0;
+}
+
+static bool restore_cpu_affinity(void)
+{
+	return pthread_setaffinity_np(pthread_self(), sizeof(saved_affinity), &saved_affinity) == 0;
+}
+#define PRESERVE_CPU_AFFINITY
+
 static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 {
 	cpuset_t cpuset;
@@ -331,6 +450,28 @@ static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 #include <pthread.h>
 #include <sched.h>
 
+INTERNAL_SCOPE cpuset_t *saved_affinity = NULL;
+
+static bool save_cpu_affinity(void)
+{
+	if (!saved_affinity)
+		saved_affinity = cpuset_create();
+
+	return pthread_getaffinity_np(pthread_self(), cpuset_size(saved_affinity), saved_affinity) == 0;
+}
+
+static bool restore_cpu_affinity(void)
+{
+	if (!saved_affinity)
+		return false;
+
+	int ret = pthread_setaffinity_np(pthread_self(), cpuset_size(saved_affinity), saved_affinity) == 0;
+	cpuset_destroy(saved_affinity);
+	saved_affinity = NULL;
+	return ret == 0;
+}
+#define PRESERVE_CPU_AFFINITY
+
 static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 {
 	cpuset_t *cpuset;
@@ -357,6 +498,18 @@ static int get_total_cpus(void)
 }
 #endif /* GET_TOTAL_CPUS_DEFINED */
 
+#ifndef PRESERVE_CPU_AFFINITY
+static bool save_cpu_affinity(void)
+{
+	return false;
+}
+
+static bool restore_cpu_affinity(void)
+{
+	return false;
+}
+#endif /* PRESERVE_CPU_AFFINITY */
+
 #ifndef SET_CPU_AFFINITY
 static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 {
@@ -381,7 +534,7 @@ static int cpuid_serialize_raw_data_internal(struct cpu_raw_data_t* single_raw,
 	/* Open file descriptor */
 	f = !strcmp(filename, "") ? stdout : fopen(filename, "wt");
 	if (!f)
-		return set_error(ERR_OPEN);
+		return cpuid_set_error(ERR_OPEN);
 	debugf(1, "Writing raw CPUID dump to '%s'\n", f == stdout ? "stdout" : filename);
 
 	/* Write raw data to output file */
@@ -427,7 +580,7 @@ static int cpuid_serialize_raw_data_internal(struct cpu_raw_data_t* single_raw,
 	/* Close file descriptor */
 	if (strcmp(filename, ""))
 		fclose(f);
-	return set_error(ERR_OK);
+	return cpuid_set_error(ERR_OK);
 }
 
 #define RAW_ASSIGN_LINE(__line) __line[EAX] = eax ; __line[EBX] = ebx ; __line[ECX] = ecx ; __line[EDX] = edx
@@ -451,7 +604,7 @@ static int cpuid_deserialize_raw_data_internal(struct cpu_raw_data_t* single_raw
 	/* Open file descriptor */
 	f = !strcmp(filename, "") ? stdin : fopen(filename, "rt");
 	if (!f)
-		return set_error(ERR_OPEN);
+		return cpuid_set_error(ERR_OPEN);
 	debugf(1, "Opening raw dump from '%s'\n", f == stdin ? "stdin" : filename);
 
 	if (use_raw_array)
@@ -564,7 +717,7 @@ static int cpuid_deserialize_raw_data_internal(struct cpu_raw_data_t* single_raw
 	/* Close file descriptor */
 	if (strcmp(filename, ""))
 		fclose(f);
-	return set_error(ERR_OK);
+	return cpuid_set_error(ERR_OK);
 }
 #undef RAW_ASSIGN_LINE
 
@@ -706,7 +859,7 @@ static int cpuid_basic_identify(struct cpu_raw_data_t* raw, struct cpu_id_t* dat
 	data->vendor = cpuid_vendor_identify(raw->basic_cpuid[0], data->vendor_str);
 
 	if (data->vendor == VENDOR_UNKNOWN)
-		return set_error(ERR_CPU_UNKN);
+		return cpuid_set_error(ERR_CPU_UNKN);
 	data->architecture = ARCHITECTURE_X86;
 	basic = raw->basic_cpuid[0][EAX];
 	if (basic >= 1) {
@@ -721,7 +874,7 @@ static int cpuid_basic_identify(struct cpu_raw_data_t* raw, struct cpu_id_t* dat
 			data->ext_family = data->family + xfamily;
 		data->ext_model = data->model + (xmodel << 4);
 	}
-	ext = raw->ext_cpuid[0][EAX] - 0x8000000;
+	ext = raw->ext_cpuid[0][EAX] - 0x80000000;
 
 	/* obtain the brand string, if present: */
 	if (ext >= 4) {
@@ -737,7 +890,7 @@ static int cpuid_basic_identify(struct cpu_raw_data_t* raw, struct cpu_id_t* dat
 	}
 	load_features_common(raw, data);
 	data->total_logical_cpus = get_total_cpus();
-	return set_error(ERR_OK);
+	return cpuid_set_error(ERR_OK);
 }
 
 static void make_list_from_string(const char* csv, struct cpu_list_t* list)
@@ -750,7 +903,7 @@ static void make_list_from_string(const char* csv, struct cpu_list_t* list)
 	list->names = (char**) malloc(sizeof(char*) * n);
 	if (!list->names) { /* Memory allocation failed */
 		list->num_entries = 0;
-		set_error(ERR_NO_MEM);
+		cpuid_set_error(ERR_NO_MEM);
 		return;
 	}
 	list->num_entries = n;
@@ -759,7 +912,7 @@ static void make_list_from_string(const char* csv, struct cpu_list_t* list)
 	for (i = 0; i <= l; i++) if (i == l || csv[i] == ',') {
 		list->names[n] = (char*) malloc(i - last);
 		if (!list->names[n]) { /* Memory allocation failed */
-			set_error(ERR_NO_MEM);
+			cpuid_set_error(ERR_NO_MEM);
 			for (j = 0; j < n; j++) free(list->names[j]);
 			free(list->names);
 			list->num_entries = 0;
@@ -793,7 +946,7 @@ static bool cpu_ident_apic_id(logical_cpu_t logical_cpu, struct cpu_raw_data_t*
 			is_apic_id_supported = true;
 			break;
 		case VENDOR_UNKNOWN:
-			set_error(ERR_CPU_UNKN);
+			cpuid_set_error(ERR_CPU_UNKN);
 			/* Fall through */
 		default:
 			is_apic_id_supported = false;
@@ -872,7 +1025,7 @@ int cpuid_get_raw_data(struct cpu_raw_data_t* data)
 {
 	unsigned i;
 	if (!cpuid_present())
-		return set_error(ERR_NO_CPUID);
+		return cpuid_set_error(ERR_NO_CPUID);
 	for (i = 0; i < 32; i++)
 		cpu_exec_cpuid(i, data->basic_cpuid[i]);
 	for (i = 0; i < 32; i++)
@@ -907,21 +1060,23 @@ int cpuid_get_raw_data(struct cpu_raw_data_t* data)
 		data->amd_fn8000001dh[i][ECX] = i;
 		cpu_exec_cpuid_ext(data->amd_fn8000001dh[i]);
 	}
-	return set_error(ERR_OK);
+	return cpuid_set_error(ERR_OK);
 }
 
 int cpuid_get_all_raw_data(struct cpu_raw_data_array_t* data)
 {
-	int cur_error = set_error(ERR_OK);
-	int ret_error = set_error(ERR_OK);
+	int cur_error = cpuid_set_error(ERR_OK);
+	int ret_error = cpuid_set_error(ERR_OK);
 	logical_cpu_t logical_cpu = 0;
 	struct cpu_raw_data_t* raw_ptr = NULL;
 
 	if (data == NULL)
-		return set_error(ERR_HANDLE);
+		return cpuid_set_error(ERR_HANDLE);
+
+	bool affinity_saved = save_cpu_affinity();
 
 	cpu_raw_data_array_t_constructor(data, true);
-	while (set_cpu_affinity(logical_cpu)) {
+	while (set_cpu_affinity(logical_cpu) || logical_cpu == 0) {
 		debugf(2, "Getting raw dump for logical CPU %i\n", logical_cpu);
 		cpuid_grow_raw_data_array(data, logical_cpu + 1);
 		raw_ptr = &data->raw[logical_cpu];
@@ -931,6 +1086,9 @@ int cpuid_get_all_raw_data(struct cpu_raw_data_array_t* data)
 		logical_cpu++;
 	}
 
+	if (affinity_saved)
+		restore_cpu_affinity();
+
 	return ret_error;
 }
 
@@ -961,13 +1119,13 @@ int cpu_ident_internal(struct cpu_raw_data_t* raw, struct cpu_id_t* data, struct
 	struct cpu_raw_data_t myraw;
 	if (!raw) {
 		if ((r = cpuid_get_raw_data(&myraw)) < 0)
-			return set_error(r);
+			return cpuid_set_error(r);
 		raw = &myraw;
 	}
 	cpu_id_t_constructor(data);
 	memset(internal->cache_mask, 0, sizeof(internal->cache_mask));
 	if ((r = cpuid_basic_identify(raw, data)) < 0)
-		return set_error(r);
+		return cpuid_set_error(r);
 	switch (data->vendor) {
 		case VENDOR_INTEL:
 			r = cpuid_identify_intel(raw, data, internal);
@@ -983,7 +1141,7 @@ int cpu_ident_internal(struct cpu_raw_data_t* raw, struct cpu_id_t* data, struct
 	/* - Deprecated since v0.5.0 */
 	data->l1_assoc     = data->l1_data_assoc;
 	data->l1_cacheline = data->l1_data_cacheline;
-	return set_error(r);
+	return cpuid_set_error(r);
 }
 
 static cpu_purpose_t cpu_ident_purpose(struct cpu_raw_data_t* raw)
@@ -994,11 +1152,14 @@ static cpu_purpose_t cpu_ident_purpose(struct cpu_raw_data_t* raw)
 
 	vendor = cpuid_vendor_identify(raw->basic_cpuid[0], vendor_str);
 	if (vendor == VENDOR_UNKNOWN) {
-		set_error(ERR_CPU_UNKN);
+		cpuid_set_error(ERR_CPU_UNKN);
 		return purpose;
 	}
 
 	switch (vendor) {
+		case VENDOR_AMD:
+			purpose = cpuid_identify_purpose_amd(raw);
+			break;
 		case VENDOR_INTEL:
 			purpose = cpuid_identify_purpose_intel(raw);
 			break;
@@ -1072,8 +1233,8 @@ static void update_cache_instances(struct internal_cache_instances_t* caches,
 
 int cpu_identify_all(struct cpu_raw_data_array_t* raw_array, struct system_id_t* system)
 {
-	int cur_error = set_error(ERR_OK);
-	int ret_error = set_error(ERR_OK);
+	int cur_error = cpuid_set_error(ERR_OK);
+	int ret_error = cpuid_set_error(ERR_OK);
 	double smt_divisor;
 	bool is_new_cpu_type;
 	bool is_last_item;
@@ -1093,10 +1254,10 @@ int cpu_identify_all(struct cpu_raw_data_array_t* raw_array, struct system_id_t*
 	struct internal_cache_instances_t caches_type, caches_all;
 
 	if (system == NULL)
-		return set_error(ERR_HANDLE);
+		return cpuid_set_error(ERR_HANDLE);
 	if (!raw_array) {
 		if ((ret_error = cpuid_get_all_raw_data(&my_raw_array)) < 0)
-			return set_error(ret_error);
+			return cpuid_set_error(ret_error);
 		raw_array = &my_raw_array;
 	}
 	system_id_t_constructor(system);
@@ -1163,7 +1324,7 @@ int cpu_identify_all(struct cpu_raw_data_array_t* raw_array, struct system_id_t*
 				switch (event) {
 					case EVENT_NEW_CPU_TYPE: cpu_type_index = system->num_cpu_types - 2; break;
 					case EVENT_LAST_ITEM:    cpu_type_index = system->num_cpu_types - 1; break;
-					default: warnf("Warning: event %i in cpu_identify_all() not handled.\n", event); return set_error(ERR_NOT_IMP);
+					default: warnf("Warning: event %i in cpu_identify_all() not handled.\n", event); return cpuid_set_error(ERR_NOT_IMP);
 				}
 				copy_affinity_mask(&system->cpu_types[cpu_type_index].affinity_mask, &affinity_mask);
 				if (event != EVENT_LAST_ITEM) {
@@ -1224,18 +1385,18 @@ int cpu_request_core_type(cpu_purpose_t purpose, struct cpu_raw_data_array_t* ra
 
 	if (!raw_array) {
 		if ((error = cpuid_get_all_raw_data(&my_raw_array)) < 0)
-			return set_error(error);
+			return cpuid_set_error(error);
 		raw_array = &my_raw_array;
 	}
 
 	for (logical_cpu = 0; logical_cpu < raw_array->num_raw; logical_cpu++) {
 		if (cpu_ident_purpose(&raw_array->raw[logical_cpu]) == purpose) {
 			cpu_ident_internal(&raw_array->raw[logical_cpu], data, &throwaway);
-			return set_error(ERR_OK);
+			return cpuid_set_error(ERR_OK);
 		}
 	}
 
-	return set_error(ERR_NOT_FOUND);
+	return cpuid_set_error(ERR_NOT_FOUND);
 }
 
 const char* cpu_architecture_str(cpu_architecture_t architecture)
@@ -1487,7 +1648,7 @@ cpu_vendor_t cpuid_get_vendor(void)
 
 	if(vendor == VENDOR_UNKNOWN) {
 		if (!cpuid_present())
-			set_error(ERR_NO_CPUID);
+			cpuid_set_error(ERR_NO_CPUID);
 		else {
 			cpu_exec_cpuid(0, raw_vendor);
 			vendor = cpuid_vendor_identify(raw_vendor, vendor_str);
@@ -1532,7 +1693,7 @@ void cpuid_get_cpu_list(cpu_vendor_t vendor, struct cpu_list_t* list)
 			break;
 		default:
 			warnf("Unknown vendor passed to cpuid_get_cpu_list()\n");
-			set_error(ERR_INVRANGE);
+			cpuid_set_error(ERR_INVRANGE);
 			list->num_entries = 0;
 			list->names = NULL;
 			break;
